@@ -182,7 +182,7 @@ class Server(IServer):
                 self._pending_who.popleft()
                 await self._next_who()
 
-        elif (line.command == ERR_NICKNAMEINUSE and
+        elif (line.command in {ERR_NICKNAMEINUSE, ERR_ERRONEUSNICKNAME} and
                 not self.registered):
             if self._alt_nicks:
                 nick = self._alt_nicks.pop(0)
@@ -554,7 +554,7 @@ class Server(IServer):
                         for symbol in symbols:
                             mode = self.isupport.prefix.from_prefix(symbol)
                             if mode is not None:
-                                channel_user.modes.append(mode)
+                                channel_user.modes.add(mode)
 
                         obj.channels.append(channel_user)
                 elif line.command == RPL_ENDOFWHOIS:
